import { Subscription } from '@essential-projects/event_aggregator_contracts';
import { IHttpClient } from '@essential-projects/http_contracts';
import { IIdentity } from '@essential-projects/iam_contracts';
import { DataModels, IManagementApiAccessor, IManagementSocketIoAccessor, Messages } from '@process-engine/management_api_contracts';
export declare class ExternalAccessor implements IManagementApiAccessor, IManagementSocketIoAccessor {
    config: any;
    private baseUrl;
    private socketCollection;
    private subscriptionCollection;
    private httpClient;
    constructor(httpClient: IHttpClient);
    initializeSocket(identity: IIdentity): void;
    disconnectSocket(identity: IIdentity): void;
    onActivityReached(identity: IIdentity, callback: Messages.CallbackTypes.OnActivityReachedCallback, subscribeOnce?: boolean): Promise<any>;
    onActivityFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnActivityFinishedCallback, subscribeOnce?: boolean): Promise<any>;
    onCallActivityWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnCallActivityWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onCallActivityFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnCallActivityFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onEmptyActivityWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnEmptyActivityWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onEmptyActivityFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnEmptyActivityFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onEmptyActivityForIdentityWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnEmptyActivityWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onEmptyActivityForIdentityFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnEmptyActivityFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onUserTaskWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnUserTaskWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onUserTaskFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnUserTaskFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onUserTaskForIdentityWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnUserTaskWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onUserTaskForIdentityFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnUserTaskFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onBoundaryEventTriggered(identity: IIdentity, callback: Messages.CallbackTypes.OnBoundaryEventTriggeredCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onIntermediateThrowEventTriggered(identity: IIdentity, callback: Messages.CallbackTypes.OnIntermediateThrowEventTriggeredCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onIntermediateCatchEventReached(identity: IIdentity, callback: Messages.CallbackTypes.OnIntermediateCatchEventReachedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onIntermediateCatchEventFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnIntermediateCatchEventFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onManualTaskWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnManualTaskWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onManualTaskFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnManualTaskFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onManualTaskForIdentityWaiting(identity: IIdentity, callback: Messages.CallbackTypes.OnManualTaskWaitingCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onManualTaskForIdentityFinished(identity: IIdentity, callback: Messages.CallbackTypes.OnManualTaskFinishedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onProcessStarted(identity: IIdentity, callback: Messages.CallbackTypes.OnProcessStartedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onProcessWithProcessModelIdStarted(identity: IIdentity, callback: Messages.CallbackTypes.OnProcessStartedCallback, processModelId: string, subscribeOnce?: boolean): Promise<Subscription>;
    onProcessTerminated(identity: IIdentity, callback: Messages.CallbackTypes.OnProcessTerminatedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onProcessError(identity: IIdentity, callback: Messages.CallbackTypes.OnProcessErrorCallback, subscribeOnce?: boolean): Promise<Subscription>;
    onProcessEnded(identity: IIdentity, callback: Messages.CallbackTypes.OnProcessEndedCallback, subscribeOnce?: boolean): Promise<Subscription>;
    removeSubscription(identity: IIdentity, subscription: Subscription): Promise<void>;
    getAllActiveCronjobs(identity: IIdentity): Promise<Array<DataModels.Cronjobs.CronjobConfiguration>>;
    getCronjobExecutionHistoryForProcessModel(identity: IIdentity, processModelId: string, startEventId?: string): Promise<Array<DataModels.Cronjobs.CronjobHistoryEntry>>;
    getCronjobExecutionHistoryForCrontab(identity: IIdentity, crontab: string): Promise<Array<DataModels.Cronjobs.CronjobHistoryEntry>>;
    getAllCorrelations(identity: IIdentity): Promise<Array<DataModels.Correlations.Correlation>>;
    getActiveCorrelations(identity: IIdentity): Promise<Array<DataModels.Correlations.Correlation>>;
    getCorrelationById(identity: IIdentity, correlationId: string): Promise<DataModels.Correlations.Correlation>;
    getCorrelationByProcessInstanceId(identity: IIdentity, processInstanceId: string): Promise<DataModels.Correlations.Correlation>;
    getCorrelationsByProcessModelId(identity: IIdentity, processModelId: string): Promise<Array<DataModels.Correlations.Correlation>>;
    getProcessModels(identity: IIdentity): Promise<DataModels.ProcessModels.ProcessModelList>;
    getProcessModelById(identity: IIdentity, processModelId: string): Promise<DataModels.ProcessModels.ProcessModel>;
    getProcessModelByProcessInstanceId(identity: IIdentity, processInstanceId: string): Promise<DataModels.ProcessModels.ProcessModel>;
    startProcessInstance(identity: IIdentity, processModelId: string, payload: DataModels.ProcessModels.ProcessStartRequestPayload, startCallbackType?: DataModels.ProcessModels.StartCallbackType, startEventId?: string, endEventId?: string): Promise<DataModels.ProcessModels.ProcessStartResponsePayload>;
    getStartEventsForProcessModel(identity: IIdentity, processModelId: string): Promise<DataModels.Events.EventList>;
    updateProcessDefinitionsByName(identity: IIdentity, name: string, payload: DataModels.ProcessModels.UpdateProcessDefinitionsRequestPayload): Promise<void>;
    deleteProcessDefinitionsByProcessModelId(identity: IIdentity, processModelId: string): Promise<void>;
    getWaitingEventsForProcessModel(identity: IIdentity, processModelId: string): Promise<DataModels.Events.EventList>;
    getWaitingEventsForCorrelation(identity: IIdentity, correlationId: string): Promise<DataModels.Events.EventList>;
    getWaitingEventsForProcessModelInCorrelation(identity: IIdentity, processModelId: string, correlationId: string): Promise<DataModels.Events.EventList>;
    triggerMessageEvent(identity: IIdentity, messageName: string, payload?: DataModels.Events.EventTriggerPayload): Promise<void>;
    triggerSignalEvent(identity: IIdentity, signalName: string, payload?: DataModels.Events.EventTriggerPayload): Promise<void>;
    getEmptyActivitiesForProcessModel(identity: IIdentity, processModelId: string): Promise<DataModels.EmptyActivities.EmptyActivityList>;
    getEmptyActivitiesForProcessInstance(identity: IIdentity, processInstanceId: string): Promise<DataModels.EmptyActivities.EmptyActivityList>;
    getEmptyActivitiesForCorrelation(identity: IIdentity, correlationId: string): Promise<DataModels.EmptyActivities.EmptyActivityList>;
    getEmptyActivitiesForProcessModelInCorrelation(identity: IIdentity, processModelId: string, correlationId: string): Promise<DataModels.EmptyActivities.EmptyActivityList>;
    finishEmptyActivity(identity: IIdentity, processInstanceId: string, correlationId: string, emptyActivityInstanceId: string): Promise<void>;
    getFlowNodeInstancesForProcessInstance(identity: IIdentity, processInstanceId: string): Promise<Array<DataModels.FlowNodeInstances.FlowNodeInstance>>;
    getUserTasksForProcessModel(identity: IIdentity, processModelId: string): Promise<DataModels.UserTasks.UserTaskList>;
    getUserTasksForProcessInstance(identity: IIdentity, processInstanceId: string): Promise<DataModels.UserTasks.UserTaskList>;
    getUserTasksForCorrelation(identity: IIdentity, correlationId: string): Promise<DataModels.UserTasks.UserTaskList>;
    getUserTasksForProcessModelInCorrelation(identity: IIdentity, processModelId: string, correlationId: string): Promise<DataModels.UserTasks.UserTaskList>;
    finishUserTask(identity: IIdentity, processInstanceId: string, correlationId: string, userTaskInstanceId: string, userTaskResult: DataModels.UserTasks.UserTaskResult): Promise<void>;
    getManualTasksForProcessModel(identity: IIdentity, processModelId: string): Promise<DataModels.ManualTasks.ManualTaskList>;
    getManualTasksForProcessInstance(identity: IIdentity, processInstanceId: string): Promise<DataModels.ManualTasks.ManualTaskList>;
    getManualTasksForCorrelation(identity: IIdentity, correlationId: string): Promise<DataModels.ManualTasks.ManualTaskList>;
    getManualTasksForProcessModelInCorrelation(identity: IIdentity, processModelId: string, correlationId: string): Promise<DataModels.ManualTasks.ManualTaskList>;
    finishManualTask(identity: IIdentity, processInstanceId: string, correlationId: string, manualTaskInstanceId: string): Promise<void>;
    getRuntimeInformationForProcessModel(identity: IIdentity, processModelId: string): Promise<Array<DataModels.Kpi.FlowNodeRuntimeInformation>>;
    getRuntimeInformationForFlowNode(identity: IIdentity, processModelId: string, flowNodeId: string): Promise<DataModels.Kpi.FlowNodeRuntimeInformation>;
    getActiveTokensForProcessModel(identity: IIdentity, processModelId: string): Promise<Array<DataModels.Kpi.ActiveToken>>;
    getActiveTokensForProcessInstance(identity: IIdentity, processInstanceId: string): Promise<Array<DataModels.Kpi.ActiveToken>>;
    getActiveTokensForCorrelationAndProcessModel(identity: IIdentity, correlationId: string, processModelId: string): Promise<Array<DataModels.Kpi.ActiveToken>>;
    getActiveTokensForFlowNode(identity: IIdentity, flowNodeId: string): Promise<Array<DataModels.Kpi.ActiveToken>>;
    getProcessModelLog(identity: IIdentity, processModelId: string, correlationId?: string): Promise<Array<DataModels.Logging.LogEntry>>;
    getProcessInstanceLog(identity: IIdentity, processModelId: string, processInstanceId: string): Promise<Array<DataModels.Logging.LogEntry>>;
    getTokensForFlowNode(identity: IIdentity, correlationId: string, processModelId: string, flowNodeId: string): Promise<Array<DataModels.TokenHistory.TokenHistoryEntry>>;
    getTokensForFlowNodeByProcessInstanceId(identity: IIdentity, processInstanceId: string, flowNodeId: string): Promise<DataModels.TokenHistory.TokenHistoryGroup>;
    getTokensForCorrelationAndProcessModel(identity: IIdentity, correlationId: string, processModelId: string): Promise<DataModels.TokenHistory.TokenHistoryGroup>;
    getTokensForProcessInstance(identity: IIdentity, processInstanceId: string): Promise<DataModels.TokenHistory.TokenHistoryGroup>;
    terminateProcessInstance(identity: IIdentity, processInstanceId: string): Promise<void>;
    private buildStartProcessInstanceUrl;
    private createRequestAuthHeaders;
    private applyBaseUrl;
    private createSocketIoSubscription;
    private createSocketForIdentity;
    private removeSocketForIdentity;
    private getSocketForIdentity;
}
