define(["require", "exports", "loggerhythm", "@essential-projects/errors_ts", "@process-engine/management_api_contracts"], function (require, exports, loggerhythm_1, EssentialProjectErrors, management_api_contracts_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const logger = loggerhythm_1.Logger.createLogger('processengine:management_api:client');
    class ManagementApiClientService {
        constructor(managementApiAccessor) {
            this.managementApiAccessor = undefined;
            this.managementApiAccessor = managementApiAccessor;
        }
        // Notifications
        async onActivityReached(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onActivityReached(identity, callback, subscribeOnce);
        }
        async onActivityFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onActivityFinished(identity, callback, subscribeOnce);
        }
        // ------------ For backwards compatibility only
        async onCallActivityWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            logger.warn('"onCallActivityWaiting" is deprecated and will be removed with the next major release! Use "onActivityReached" instead!');
            return this.managementApiAccessor.onCallActivityWaiting(identity, callback, subscribeOnce);
        }
        async onCallActivityFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            logger.warn('"onCallActivityFinished" is deprecated and will be removed with the next major release! Use "onActivityFinished" instead!');
            return this.managementApiAccessor.onCallActivityFinished(identity, callback, subscribeOnce);
        }
        // ------------
        async onEmptyActivityWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onEmptyActivityWaiting(identity, callback, subscribeOnce);
        }
        async onEmptyActivityFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onEmptyActivityFinished(identity, callback, subscribeOnce);
        }
        async onEmptyActivityForIdentityWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onEmptyActivityForIdentityWaiting(identity, callback, subscribeOnce);
        }
        async onEmptyActivityForIdentityFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onEmptyActivityForIdentityFinished(identity, callback, subscribeOnce);
        }
        async onUserTaskWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onUserTaskWaiting(identity, callback, subscribeOnce);
        }
        async onUserTaskFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onUserTaskFinished(identity, callback, subscribeOnce);
        }
        async onUserTaskForIdentityWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onUserTaskForIdentityWaiting(identity, callback, subscribeOnce);
        }
        async onUserTaskForIdentityFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onUserTaskForIdentityFinished(identity, callback, subscribeOnce);
        }
        async onBoundaryEventTriggered(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onBoundaryEventTriggered(identity, callback, subscribeOnce);
        }
        async onIntermediateThrowEventTriggered(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onIntermediateThrowEventTriggered(identity, callback, subscribeOnce);
        }
        async onIntermediateCatchEventReached(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onIntermediateCatchEventReached(identity, callback, subscribeOnce);
        }
        async onIntermediateCatchEventFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onIntermediateCatchEventFinished(identity, callback, subscribeOnce);
        }
        async onManualTaskWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onManualTaskWaiting(identity, callback, subscribeOnce);
        }
        async onManualTaskFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onManualTaskFinished(identity, callback, subscribeOnce);
        }
        async onManualTaskForIdentityWaiting(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onManualTaskForIdentityWaiting(identity, callback, subscribeOnce);
        }
        async onManualTaskForIdentityFinished(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onManualTaskForIdentityFinished(identity, callback, subscribeOnce);
        }
        async onProcessStarted(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onProcessStarted(identity, callback, subscribeOnce);
        }
        async onProcessWithProcessModelIdStarted(identity, callback, processModelId, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onProcessWithProcessModelIdStarted(identity, callback, processModelId, subscribeOnce);
        }
        async onProcessTerminated(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onProcessTerminated(identity, callback, subscribeOnce);
        }
        async onProcessError(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onProcessError(identity, callback, subscribeOnce);
        }
        async onProcessEnded(identity, callback, subscribeOnce = false) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.onProcessEnded(identity, callback, subscribeOnce);
        }
        async removeSubscription(identity, subscription) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.removeSubscription(identity, subscription);
        }
        // Cronjobs
        async getAllActiveCronjobs(identity) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getAllActiveCronjobs(identity);
        }
        async getCronjobExecutionHistoryForProcessModel(identity, processModelId, startEventId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getCronjobExecutionHistoryForProcessModel(identity, processModelId, startEventId);
        }
        async getCronjobExecutionHistoryForCrontab(identity, crontab) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getCronjobExecutionHistoryForCrontab(identity, crontab);
        }
        // Correlations
        async getAllCorrelations(identity) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getAllCorrelations(identity);
        }
        async getActiveCorrelations(identity) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getActiveCorrelations(identity);
        }
        async getCorrelationById(identity, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getCorrelationById(identity, correlationId);
        }
        async getCorrelationByProcessInstanceId(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getCorrelationByProcessInstanceId(identity, processInstanceId);
        }
        async getCorrelationsByProcessModelId(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getCorrelationsByProcessModelId(identity, processModelId);
        }
        // ProcessModels
        async getProcessModels(identity) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getProcessModels(identity);
        }
        async getProcessModelById(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getProcessModelById(identity, processModelId);
        }
        async getProcessModelByProcessInstanceId(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getProcessModelByProcessInstanceId(identity, processInstanceId);
        }
        async startProcessInstance(identity, processModelId, payload, startCallbackType = management_api_contracts_1.DataModels.ProcessModels.StartCallbackType.CallbackOnProcessInstanceCreated, startEventId, endEventId) {
            this.ensureIsAuthorized(identity);
            if (!Object.values(management_api_contracts_1.DataModels.ProcessModels.StartCallbackType).includes(startCallbackType)) {
                throw new EssentialProjectErrors.BadRequestError(`${startCallbackType} is not a valid return option!`);
            }
            if (startCallbackType === management_api_contracts_1.DataModels.ProcessModels.StartCallbackType.CallbackOnEndEventReached && !endEventId) {
                throw new EssentialProjectErrors.BadRequestError('Must provide an EndEventId, when using callback type \'CallbackOnEndEventReached\'!');
            }
            return this.managementApiAccessor.startProcessInstance(identity, processModelId, payload, startCallbackType, startEventId, endEventId);
        }
        async getStartEventsForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getStartEventsForProcessModel(identity, processModelId);
        }
        async updateProcessDefinitionsByName(identity, name, payload) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.updateProcessDefinitionsByName(identity, name, payload);
        }
        async deleteProcessDefinitionsByProcessModelId(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.deleteProcessDefinitionsByProcessModelId(identity, processModelId);
        }
        // Events
        async getWaitingEventsForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getWaitingEventsForProcessModel(identity, processModelId);
        }
        async getWaitingEventsForCorrelation(identity, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getWaitingEventsForCorrelation(identity, correlationId);
        }
        async getWaitingEventsForProcessModelInCorrelation(identity, processModelId, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getWaitingEventsForProcessModelInCorrelation(identity, processModelId, correlationId);
        }
        async triggerMessageEvent(identity, messageName, payload) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.triggerMessageEvent(identity, messageName, payload);
        }
        async triggerSignalEvent(identity, signalName, payload) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.triggerSignalEvent(identity, signalName, payload);
        }
        // Empty Activities
        async getEmptyActivitiesForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getEmptyActivitiesForProcessModel(identity, processModelId);
        }
        async getEmptyActivitiesForProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getEmptyActivitiesForProcessInstance(identity, processInstanceId);
        }
        async getEmptyActivitiesForCorrelation(identity, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getEmptyActivitiesForCorrelation(identity, correlationId);
        }
        async getEmptyActivitiesForProcessModelInCorrelation(identity, processModelId, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getEmptyActivitiesForProcessModelInCorrelation(identity, processModelId, correlationId);
        }
        async finishEmptyActivity(identity, processInstanceId, correlationId, emptyActivityInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.finishEmptyActivity(identity, processInstanceId, correlationId, emptyActivityInstanceId);
        }
        // FlowNodeInstances
        async getFlowNodeInstancesForProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getFlowNodeInstancesForProcessInstance(identity, processInstanceId);
        }
        // UserTasks
        async getUserTasksForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getUserTasksForProcessModel(identity, processModelId);
        }
        async getUserTasksForProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getUserTasksForProcessInstance(identity, processInstanceId);
        }
        async getUserTasksForCorrelation(identity, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getUserTasksForCorrelation(identity, correlationId);
        }
        async getUserTasksForProcessModelInCorrelation(identity, processModelId, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getUserTasksForProcessModelInCorrelation(identity, processModelId, correlationId);
        }
        async finishUserTask(identity, processInstanceId, correlationId, userTaskInstanceId, userTaskResult) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.finishUserTask(identity, processInstanceId, correlationId, userTaskInstanceId, userTaskResult);
        }
        // ManualTasks
        async getManualTasksForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getManualTasksForProcessModel(identity, processModelId);
        }
        async getManualTasksForProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getManualTasksForProcessInstance(identity, processInstanceId);
        }
        async getManualTasksForCorrelation(identity, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getManualTasksForCorrelation(identity, correlationId);
        }
        async getManualTasksForProcessModelInCorrelation(identity, processModelId, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getManualTasksForProcessModelInCorrelation(identity, processModelId, correlationId);
        }
        async finishManualTask(identity, processInstanceId, correlationId, manualTaskInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.finishManualTask(identity, processInstanceId, correlationId, manualTaskInstanceId);
        }
        // Heatmap related features
        async getRuntimeInformationForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getRuntimeInformationForProcessModel(identity, processModelId);
        }
        async getRuntimeInformationForFlowNode(identity, processModelId, flowNodeId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getRuntimeInformationForFlowNode(identity, processModelId, flowNodeId);
        }
        async getActiveTokensForProcessModel(identity, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getActiveTokensForProcessModel(identity, processModelId);
        }
        async getActiveTokensForCorrelationAndProcessModel(identity, correlationId, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getActiveTokensForCorrelationAndProcessModel(identity, correlationId, processModelId);
        }
        async getActiveTokensForProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getActiveTokensForProcessInstance(identity, processInstanceId);
        }
        async getActiveTokensForFlowNode(identity, flowNodeId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getActiveTokensForFlowNode(identity, flowNodeId);
        }
        async getProcessModelLog(identity, processModelId, correlationId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getProcessModelLog(identity, processModelId, correlationId);
        }
        async getProcessInstanceLog(identity, processModelId, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getProcessInstanceLog(identity, processModelId, processInstanceId);
        }
        async getTokensForFlowNode(identity, correlationId, processModelId, flowNodeId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getTokensForFlowNode(identity, correlationId, processModelId, flowNodeId);
        }
        async getTokensForFlowNodeByProcessInstanceId(identity, processInstanceId, flowNodeId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getTokensForFlowNodeByProcessInstanceId(identity, processInstanceId, flowNodeId);
        }
        async getTokensForCorrelationAndProcessModel(identity, correlationId, processModelId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getTokensForCorrelationAndProcessModel(identity, correlationId, processModelId);
        }
        async getTokensForProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.getTokensForProcessInstance(identity, processInstanceId);
        }
        async terminateProcessInstance(identity, processInstanceId) {
            this.ensureIsAuthorized(identity);
            return this.managementApiAccessor.terminateProcessInstance(identity, processInstanceId);
        }
        ensureIsAuthorized(identity) {
            const authTokenNotProvided = !identity || typeof identity.token !== 'string';
            if (authTokenNotProvided) {
                throw new EssentialProjectErrors.UnauthorizedError('No auth token provided!');
            }
        }
    }
    exports.ManagementApiClientService = ManagementApiClientService;
});
//# sourceMappingURL=management_api_client_service.js.map